module detector(	    i_sys_rst,
    i_clk_100_bus,
	o_M_abort,
    o_M_ABus,
    o_M_BE,
    o_M_busLock,
    o_M_TAttribute,
    o_M_lockErr,
    o_M_mSize,
    o_M_priority,
    o_M_rdBurst,
    o_M_request,
    o_M_RNW,
    o_M_size,
    o_M_type,
    o_M_wrBurst,
    o_M_wrDBus,
	i_MPMC_Done_Init,
	i_PLB_MAddrAck,
    i_PLB_MBusy,
    i_PLB_MRdErr,
    i_PLB_MWrErr,
    i_PLB_MRdBTerm,
    i_PLB_MRdDAck,
    i_PLB_MRdDBus,
    i_PLB_MRdWdAddr,
    i_PLB_MRearbitrate,
    i_PLB_MSSize,
    i_PLB_MWrBTerm,
    i_PLB_MWrDAck,
    i_PLB_MTimeout,
    i_PLB_PAValid,
	i_Sl_addrAck,
    i_Sl_wrDAck,
    i_Sl_wrComp,
	o_Recog
	);
	
	////////////////////////////////////////////////////////////
	////////////////////////// PORTS ///////////////////////////
	////////////////////////////////////////////////////////////

	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Assigned by Xilinx Tools
	parameter   					C_PLBV46_NUM_SLAVES 	= 1;
	parameter   					C_PLBV46_DWIDTH     	= 64;
	
	// Inherited From MPD file
	parameter   					C_VIDEO_RAM_BASEADDR	= 32'h00000000;
	parameter						C_BYTES_PER_LINE		= 4096;

	// Local
	localparam  					C_BE_BITS           	= (C_PLBV46_DWIDTH / 8);
	
	localparam						C_BURST_LENGTH			= 16;	// Always write 16 times per request (maximum per burst)
	localparam						C_BYTES_PER_BURST		= C_BURST_LENGTH * 4;
	localparam						C_LINES_PER_FRAME		= 480;
	localparam						C_PIXELS_PER_LINE		= 720;
	localparam						C_BURSTS_PER_LINE 		= C_PIXELS_PER_LINE / C_BURST_LENGTH;

    //FSM States
	localparam						C_STATE_BITS 			= 3;
	localparam	[C_STATE_BITS-1:0]	S_INIT 			        = 0,
									S_START_LINE			= 1,
									S_WRITE_REQUEST			= 2,
									S_WRITE					= 3,	
									S_WRITE_COMPLETE		= 4;
									
	localparam	[C_STATE_BITS-1:0]	R_INIT 			        = 0,
									R_START_LINE			= 1,
									R_READ_REQUEST			= 2,
									R_READ					= 3,	
									R_READ_COMPLETE			= 4,
									R_POST_READ_PROCESS		= 5;
									
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////									
		// System
	input                           		i_sys_rst;
	input                           		i_clk_100_bus;
	
    //MPMC
    input                                   i_MPMC_Done_Init;
	
		// PLB Master
	output                                  o_M_abort;
	output  [0:31]                          o_M_ABus;
	output  [0:(C_BE_BITS-1)]               o_M_BE;
	output                                  o_M_busLock;
	output  [0:15]                          o_M_TAttribute;
	output                                  o_M_lockErr;
	output  [0:1]                           o_M_mSize;
	output  [0:1]                           o_M_priority;
	output                                  o_M_rdBurst;
	output                                  o_M_request;
	output                                  o_M_RNW;
	output  [0:3]                           o_M_size;
	output  [0:2]                           o_M_type;
	output                                  o_M_wrBurst;
	output  [0:(C_PLBV46_DWIDTH-1)]         o_M_wrDBus;		
	input                                   i_PLB_MAddrAck;
	input                                   i_PLB_MBusy;
	input                                   i_PLB_MRdErr;
	input                                   i_PLB_MWrErr;
	input                                   i_PLB_MRdBTerm;
	input                                   i_PLB_MRdDAck;
	input   [0:(C_PLBV46_DWIDTH-1)]         i_PLB_MRdDBus;
	input   [0:3]                           i_PLB_MRdWdAddr;
	input                                   i_PLB_MRearbitrate;
	input   [0:1]                           i_PLB_MSSize;
	input                                   i_PLB_MWrBTerm;
	input                                   i_PLB_MWrDAck;
	input                                   i_PLB_MTimeout;
	input                                   i_PLB_PAValid;
	input   [0:(C_PLBV46_NUM_SLAVES-1)]     i_Sl_addrAck;
	input   [0:(C_PLBV46_NUM_SLAVES-1)]     i_Sl_wrDAck;
	input   [0:(C_PLBV46_NUM_SLAVES-1)]     i_Sl_wrComp;
	input	[0:31]							o_Recog;
	
		// Memory Write State Machine
	reg		[C_STATE_BITS-1:0]		c_ns;
	
	wire		[0:31]					r_M_ABus;
	reg     [0:(C_PLBV46_DWIDTH-1)]	c_M_wrDBus;
	wire    	w_M_request;
	wire		r_M_request;
	reg 	init_done;
	reg 	[31:0]	count;
	reg		[1:0]	count_n;
	reg		[C_STATE_BITS-1:0]	state;
	reg		[C_STATE_BITS-1:0]	r_state;
	
	reg		[31:0]				readData;
	reg		r_nw;
	reg		[31:0]				r_count, w_count, x_cord, y_cord;
	reg		[31:0]				processData;
	reg		[31:0]				writeData;
	reg		[31:0]				x_cord_sum, y_cord_sum, detected_counter;
	
	reg		[31:0]				color_min;
	reg		[31:0]				color_max;
	reg		[31:0]				color_replaced;
	
	wire	[31:0]				readAddress, writeAddress;
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
	    // Output to PLB Bus
	assign 	o_M_abort				= 1'b0;						// Not Used
	assign	o_M_ABus				= r_M_ABus;
	assign	o_M_BE					= 8'hFF;		// Burst Length
	assign	o_M_busLock				= 1'b0;						// Not Used
	assign	o_M_TAttribute			= 16'b0;					// Not Used
	assign 	o_M_lockErr				= 1'b0;						// Not Used
	assign 	o_M_mSize				= 2'b0;						// 32-bit bus size
	assign 	o_M_priority			= 2'b11;					// Always highest priority
	assign 	o_M_rdBurst				= 1'b0;						// Never requesting read operations
	
	assign 	o_M_request				= r_nw ? r_M_request : w_M_request;		
	assign 	o_M_RNW					= r_nw;						// Never reading, always writing
	
	assign 	o_M_size				= 4'hA;						// Burst of Word Size (32-bit) Transfers
	assign 	o_M_type				= 3'b000;					// Standard Memory Transfer
	assign	o_M_wrBurst				= 1'b0;		
	
	assign 	o_M_wrDBus				= writeData;
	
	assign	w_M_request				= ( state == S_WRITE_REQUEST ); //request = 1 if state is in write_request
	assign	r_M_request				= (	r_state == R_READ_REQUEST); //request = 1 if state is in read_request
	
	assign 	readAddress = 32'h40000000 + r_count * 4; //The custom logic reads from 0x40000000 to 0x40200000
	assign	writeAddress = 32'h49000000 + w_count * 4; //The custom logic writes from 0x49000000 to 0x49200000
	
	assign 	r_M_ABus = r_nw ? readAddress: writeAddress;
	
	// This is the Write FSM, It is written with reference to the PLB bus timing diagrams included in the appendix of our report
	always @(posedge i_clk_100_bus or posedge i_sys_rst)
	begin
		if ( i_sys_rst == 1'b1 ) //reset
		begin
			state = S_INIT;
			w_count = 0;
		end
		else begin
			case (state) 
		
			S_INIT: //the reset state, if MPMC is ready and write operation is needed, the state transitions to request state
			begin
				if (i_MPMC_Done_Init == 1'b1 && r_nw == 0)
				begin
					state = S_WRITE_REQUEST;
				end
				else
				begin
					state = S_INIT;
				end
			end
		
			S_WRITE_REQUEST:
			begin
				if (i_Sl_addrAck == 1'b1 && i_PLB_PAValid == 1'b1) //If the PLB acknowledges the write request, go to the write state
				begin
					state = S_WRITE;
				end
				else
				begin
					state = S_WRITE_REQUEST;
				end
				
				if (writeAddress == 32'h49210000) //This if statement writes three internal registers of the custom logic to three memory addresses for the processor to read
				begin
					writeData = x_cord_sum;
				end
				else if (writeAddress == 32'h49210004)
				begin
					writeData = y_cord_sum;
				end
				else if (writeAddress == 32'h49210008)
				begin
					writeData = detected_counter;
				end
				else
				begin
					writeData = processData;
				end
			end
			
			S_WRITE:
			begin
				if ( i_Sl_wrComp == 1'b1 )
				begin
					state = S_WRITE_COMPLETE;
					if (w_count >= 32'h85000) 
					begin
						w_count = 0;
					end
					else
					begin
						w_count = w_count + 1;
					end
				end
				else
				begin
					state = S_WRITE;
				end
			end
		
			S_WRITE_COMPLETE:
			begin
				if (r_nw == 0)
				begin
					if (writeAddress < 32'h4920FFF0 || writeAddress > 32'h49210010) //disable write operation if the write_address is in a particular range.
					begin
						if (w_count >= 32'h85000)
						begin
							w_count = 0;
						end
						else
						begin
							w_count = w_count + 1;
						end
						state = S_WRITE_COMPLETE;
					end
					else
					begin
						state = S_WRITE_REQUEST;
					end
				end
				else
				begin
					state = S_WRITE_COMPLETE;
				end			
			end
			
			endcase
		end
	end
	
	//This is the Read FSM, It is written with reference to the PLB bus timing diagrams included in the appendix of our report
	//It is used to read every pixel of the incoming data and performs beacon detection by reading the color of each pixel
	always @(posedge i_clk_100_bus or posedge i_sys_rst)
	begin
		if ( i_sys_rst == 1'b1 ) //reset
		begin
			r_state = R_INIT;
			readData = 0;
			r_count = 0;
			detected_counter = 0;
			x_cord_sum = 0;
			y_cord_sum = 0;
			x_cord = 0;
			y_cord = 0;
		end
		else begin
			case (r_state)
		
			R_INIT: //the reset state, if MPMC is ready and read operation is needed, the state transitions to request state
			begin
				if (i_MPMC_Done_Init == 1'b1 && r_nw == 1)
				begin
					r_state = R_READ_REQUEST;
				end
				else
				begin
					r_state = R_INIT;
				end
			end
		
			R_READ_REQUEST: //If the PLB acknowledges the write request, go to the read state
			begin
				if (i_Sl_addrAck == 1'b1 && i_PLB_PAValid == 1'b1)
				begin
					r_state = R_READ;
				end
				else
				begin
					r_state = R_READ_REQUEST;
				end			
			end
			
			R_READ:
			begin
				readData = i_PLB_MRdDBus;
				if ( i_PLB_MRdDAck == 1'b1 )
				begin
					r_state = R_READ_COMPLETE;
					if (r_count >= 32'h85000) //checks the bound on the memory location, if the entire frame is read already, reset the count register.
					begin
						r_count = 0;
						x_cord = 0;
						y_cord = 0;
					end
					else
					begin
						r_count = r_count + 1;
						x_cord = x_cord + 1;
						if (x_cord >= 32'h400) //keeps track of x and y coordinate in the 1024x512 frame
						begin
							x_cord = 0;
							y_cord = y_cord + 1;
						end
					end
				end
				else
				begin
					r_state = R_READ;
				end
			end
		
			R_READ_COMPLETE: 
			begin
				if (readAddress == 32'h40210000) //Change the value of three internal registers based on three memory addresses
				begin
					color_min = readData;
				end
				else if (readAddress == 32'h40210004)
				begin
					color_max = readData;
				end
				else if (readAddress == 32'h40210008)
				begin
					color_replaced = readData;
				end
				
				if (r_nw == 1)
				begin
					r_state = R_POST_READ_PROCESS;
				end
				else
				begin
					r_state = R_READ_COMPLETE;
				end			
			end
			
			R_POST_READ_PROCESS: //post processing state
			begin 
				if (readAddress == 32'h40000000)
				begin
					detected_counter = 0;
					x_cord_sum = 0;
					y_cord_sum = 0;
				end
				if (x_cord >= 32'h3 && x_cord < 32'h27D && y_cord >= 32'h3 && y_cord < 32'h1DD && 
					readData[23:16] >= color_min[23:16] && readData[15:8] >= color_min[15:8] && readData[7:0] >= color_min[7:0] &&
					readData[23:16] <= color_max[23:16] && readData[15:8] <= color_max[15:8] && readData[7:0] <= color_max[7:0]) 
					//checks the color bound only if the x and y coordinates are within a certain range.
					//This is because there are some dummy data written by the video_to_ram IP
				begin
					processData = color_replaced;
					x_cord_sum = x_cord_sum + x_cord;
					y_cord_sum = y_cord_sum + y_cord;
					detected_counter = detected_counter + 1;
				end
				else
				begin
					processData = readData;
				end
				r_state = R_READ_REQUEST;
			end
			endcase
		end
	end
	
	always @ (*) //This is a FSM to regulate the read/write operations
	begin
		if ( i_sys_rst == 1'b1 )
		begin
			r_nw = 1;
		end
		else if (r_count == w_count)
		begin
			r_nw = 1;
		end
		else
		begin
			r_nw = 0;
		end
	end
		
endmodule
