`timescale 1ns / 1ps

module tone_gen(
SYSTEM_CLOCK,

ac97_bit_clock,	// Generated by AC'97
ac97_sdata_in,		// Serial data from AC'97

ac97_sdata_out,	// Serial data to AC'97
ac97_synch,			// Defines boundries of AC'97 frames, controls warm reset
audio_reset_b,		// AC'97 codec cold reset

TONE1_PERIOD,   //define frequency of tone, in units of clock ticks
TONE1_LENGTH,   //define duration of tone, in units of clock ticks
TONE1_BEGIN,     //Tone will play when this is set high
TONE1_DONE,

TONE2_PERIOD,   //define frequency of tone, in units of clock ticks
TONE2_LENGTH,   //define duration of tone, in units of clock ticks
TONE2_BEGIN,     //Tone will play when this is set high
TONE2_DONE
);

input SYSTEM_CLOCK, ac97_bit_clock, ac97_sdata_in;
input [31:0] TONE1_PERIOD, TONE1_LENGTH;
input TONE1_BEGIN;
input [31:0] TONE2_PERIOD, TONE2_LENGTH;
input TONE2_BEGIN;
output TONE1_DONE;
output TONE2_DONE;
output ac97_sdata_out, ac97_synch, audio_reset_b;

wire [19:0] left_in_data, right_in_data;
wire [19:0] left1_out_data, right1_out_data;
wire [19:0] left1_tone_data, right1_tone_data;
wire tone1Active;
reg [31:0] tone1Counter;
wire [19:0] left2_out_data, right2_out_data;
wire [19:0] left2_tone_data, right2_tone_data;
wire tone2Active;
reg [31:0] tone2Counter;

wire [19:0] left_out_data, right_out_data;

// instantiate the clock generation module
//CLOCK_GEN CLOCK_GEN 
//(
//SYSTEM_CLOCK,
//system_clock_buffered,
//reset
//);

assign system_clock_buffered = SYSTEM_CLOCK;

// Audio module
audio audio0 (system_clock_buffered, reset, left_in_data, left_out_data, right_in_data, right_out_data, ready,
	      audio_reset_b, ac97_sdata_out, ac97_sdata_in,
         ac97_synch, ac97_bit_clock	  
			);


// Processing module: takes input, does something, sends output				  
processing processing1 (system_clock_buffered, TONE1_PERIOD, left1_tone_data, right1_tone_data);			  
processing processing2 (system_clock_buffered, TONE2_PERIOD, left2_tone_data, right2_tone_data);

assign left_out_data = {0, left1_out_data[19:1]} + {0,left2_out_data[19:1]};
assign right_out_data = {0,right1_out_data[19:1]} + {0,right2_out_data[19:1]};

////////////////////////////////////////////////////
assign tone1Active = (tone1Counter < TONE1_LENGTH);

assign left1_out_data = tone1Active?left1_tone_data:0;
assign right1_out_data = tone1Active?right1_tone_data:0;

assign TONE1_DONE = !(tone1Active);
assign TONE2_DONE = !(tone2Active);

always @ (posedge system_clock_buffered) begin
   if (TONE1_BEGIN == 1) begin
      tone1Counter <= 0;
   end
   else begin
      if(tone1Counter >= TONE1_LENGTH) begin
         tone1Counter <= tone1Counter;
      end
      else begin
         tone1Counter <= tone1Counter + 1;
      end
   end
end

////////////////////////////////////////
assign tone2Active = (tone2Counter < TONE2_LENGTH);

assign left2_out_data = tone2Active?left2_tone_data:0;
assign right2_out_data = tone2Active?right2_tone_data:0;

always @ (posedge system_clock_buffered) begin
   if (TONE2_BEGIN == 1) begin
      tone2Counter <= 0;
   end
   else begin
      if(tone2Counter >= TONE2_LENGTH) begin
         tone2Counter <= tone2Counter;
      end
      else begin
         tone2Counter <= tone2Counter + 1;
      end
   end
end

endmodule