/********************* Include Libraries ***********************/
#include "xparameters.h"    /* generated system parameters                  */
#include "xbasic_types.h"   /* Xilinx basic types for device drivers        */
#include "xgpio.h"          /* layer 1 GPIO device driver                   */
#include "xiic_l.h"
#include "xiic.h"
#include "xtmrctr.h"        /* layer 1 timer driver                         */
#include "xintc.h"          /* layer 1 interrupt controller driver          */

#include <math.h>
#include <stdlib.h>

/********************* Defining Constants **********************/
#define VIDEO_IN_BASEADDR 0x40000000
#define VIDEO_OUT_BASEADDR 0x41000000
#define SCREEN_SWITCH_BASEADDR 0x85000000
#define VIDEO_REPLACE_ADDR 0x49000000
#define INTC_BASEADDR 0x80002000
#define TIMER_BASEADDR 0x80002100

#define COLOR_RED 0x00FF0000
#define COLOR_GREEN 0x0000FF00
#define COLOR_YELLOW 0x00AAAA00
#define COLOR_BLUE 0x000000FF
#define COLOR_WHITE 0x00FFFFFF
#define COLOR_BLACK 0x00FFFFFF

#define bool int
#define true 1
#define false 0

#define SCREEN_WIDTH 640
#define SCREEN_HEIGHT 480

#define BEACON_X_ADDR 0x49210000
#define BEACON_Y_ADDR 0x49210004
#define BEACON_COUNT_ADDR 0x49210008

#define COLOR_LOWER_ADDR 0x4020FFFC
#define COLOR_UPPER_ADDR 0x40210000
#define COLOR_REPLACE_ADDR 0X40210004

#define COLOR_LOWER_BOUND_CALI 0x00A00000
#define COLOR_UPPER_BOUND_CALI 0x00FF7070
#define COLOR_LOWER_BOUND_BEACON 0x00A00000
#define COLOR_UPPER_BOUND_BEACON 0x00FF7070

#define COLOR_REPLACED 0x0

#define printf xil_printf

//#define RAND_MAX 2147483647

// Snake
typedef struct list_elem SnakePoint;

// Video
#define GPO_REG_OFFSET 0x124
#define DECODER_ADDR (0x42>>1) //Read: 0x41, Write: 0x40
#define SEND_CNT 3
#define RECV_CNT 3
#define GPO_RESETS_OFF 1
#define GPO_RESET_IIC 3
#define GPO_RESET_DECODER 0

#define DECODER_COMP_CONFIG_CNT 18

// Audio
//Musical note frequency
#define NOTE_A 440 
#define NOTE_B 494
#define NOTE_C 262
#define NOTE_D 294
#define NOTE_E 330
#define NOTE_F 349
#define NOTE_G 392
#define NOTE_Cs 277
#define NOTE_Ds 311
#define NOTE_Fs 370
#define NOTE_Gs 415
#define NOTE_As 466

//Musical note period = 1/NOTE * CLK_FREQ
#define PERIOD_A 227273
#define PERIOD_B 211417
#define PERIOD_C 381679
#define PERIOD_D 340136
#define PERIOD_E 303030
#define PERIOD_F 286533
#define PERIOD_G 255102
#define PERIOD_Cs 361011
#define PERIOD_Ds 321543
#define PERIOD_Fs 270270
#define PERIOD_Gs 240964
#define PERIOD_As 214592

//Semi tones


//Pitches for the audio
#define PITCH_A 122726
#define PITCH_C 206106
#define PITCH_HIT 103053
#define PITCH_GAME 245452
#define PITCH_GAME_2 61363
#define AUDIO_LENGTH CLOCKS_PER_MS*500

//Tone durations
#define TONE_DUR 12000000
#define PAUSE_DUR TONE_DUR / 15
#define GAME_OVER_DUR TONE_DUR * 5

// Push Buttons
#define PUSH_BUTTON_UP 0x8
#define PUSH_BUTTON_DOWN 0x4
#define PUSH_BUTTON_LEFT 0x2
#define PUSH_BUTTON_RIGHT 0x1

// Timer
#define CLOCK_MHZ       100000000
#define CLOCKS_PER_MS   (CLOCK_MHZ / 1000)
#define TIME_SLICE_MS   1
#define RESET_VALUE     (0xFFFFFFFF - (TIME_SLICE_MS * CLOCKS_PER_MS))
#define TIMER_COUNTER_0 0

/********************* Global Variables ************************/
////////////////////// From snake.h
struct list_elem
{
	int x0;
	int y0;
	int x1;
	int y1;
	int end;
	struct list_elem* next;
};
struct VideoModule {
  Xuint8 addr;
  Xuint8 config_val;
  Xuint8 actual_val;
};

SnakePoint * headPtr;
SnakePoint * tailPtr;

int foodX,foodY;
int beaconX, beaconY;
int incrX,incrY;

//int detectorX[4]={10,630,10,630};
//int detectorY[4]={10,10,470,470};
int detectorX[4]={40,600,40,600};
int detectorY[4]={40,40,440,440};
int detectorU[4]={40,600,40,600};
int detectorV[4]={40,40,440,440};
int deltaU;
int deltaV;

Xuint32 snake_score;

struct VideoModule decoder_comp[] = { 
  { 0x00, 0x04, 0 },
  { 0x15, 0x00, 0 },
  { 0x17, 0x41, 0 },
  { 0x27, 0x58, 0 },
  { 0x3a, 0x16, 0 },
  { 0x50, 0x04, 0 },
  { 0x0e, 0x80, 0 },
  { 0x50, 0x20, 0 },
  { 0x52, 0x18, 0 },
  { 0x58, 0xed, 0 },
  { 0x77, 0xc5, 0 },
  { 0x7c, 0x93, 0 },
  { 0x7d, 0x00, 0 },
  { 0xd0, 0x48, 0 },
  { 0xd5, 0xa0, 0 },
  { 0xd7, 0xea, 0 },
  { 0xe4, 0x3e, 0 },
  { 0xea, 0x0f, 0 },
  { 0x0e, 0x00, 0 } };

//GPIO for switches and push buttons
XGpio SW_Gpio;

//GPios for audio
XGpio Audio1_Begin;
XGpio Audio1_Period;
XGpio Audio1_Length;
XGpio Audio1_Done;
XGpio Audio2_Begin;
XGpio Audio2_Period;
XGpio Audio2_Length;
XGpio Audio2_Done;

//Interrupt controller
XIntc InterruptController;

//Timer
XTmrCtr TimerCounter;
Xuint32 TimerExpiredCount;

/********************* Function Prototypes *********************/
////////////////////// From draw.h
/**
 * Utility Functions
 */
int w_max( int a,  int b);
int w_min(int a,  int b);
unsigned int w_abs(int a);

/**
 * Paint Functions
 */
void write_pixel(unsigned int value, unsigned int x, unsigned int y);
unsigned int get_pixel(unsigned int x, unsigned int y);
void clear_screen();
void red_screen();
void green_screen();
void blue_screen();

void draw_rectangle(unsigned int x1, unsigned int y1, unsigned int width, unsigned int length,  const int colour);
void draw_line(int x0, int y0, int x1, int y1,const int colour);
void draw_thickLine(int x0, int y0, int x1, int y1,int width,const int colour);
void draw_niceLine(int x0, int y0, int x1, int y1, int width, const int colour);
void draw_box(int x0, int y0,int x1,int y1,int width, const int colour);
void draw_ball(int X, int Y, int radius, int colour);

void S(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour);
void T(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour);
void A(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour);
void R(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour);
void G(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour);
void M(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour);
void E(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour);
void O(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour);
void V(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour);
		
void C(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour);
void L(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour);
void I(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour);
void B(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour);

void P(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour);		

void N(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour);		
		
void game_start();
void game_over();
void cali_start();

////////////////////// From snake.h
//Snake Functions
void generateFood();

void drawSnakePoint(SnakePoint* point, int color);
void drawSnake(SnakePoint * headPointer);
void snakeEat(int incrX, int incrY);
int detectFront(int x, int y, int deltaX, int deltaY, unsigned int color);
int detectFood(int x, int y);
int detectBeacon(int x, int y);

void setIncr(int deltaX,int deltaY,int * incrX, int* incrY);
int snakeMove(int incrX, int incrY);
void snakeInit();

int update();

//Detector 
int transformToX(int u);
int transformToY(int v);
void showDetector();
void setBeacon(int u, int v);
unsigned int getCount();
unsigned int getVSum();
unsigned int getUSum();
int calculateUV(int sum, int count);

//Others
void delayTime(Xuint32 ms_to_delay);

////////////////////// From video_setup.h
void configDecoder(struct VideoModule *decoder, int config_cnt );

////////////////////// From audio_setup.h
void makeTone(int channel, u32 pitch, u32 length, int async);
void play_gamestart();
void play_score();
void play_gameover();


//////////////////////Play game

void play();

////////////////////// From timer_setup.h
void TimerCounterHandler(void *CallBackRef, Xuint8 TimerCounterNumber);

int main()
{
//	Xuint8 start_addr = 0;
//	Xuint8 send_data[SEND_CNT] = {0};
//	Xuint32 send_cnt;
//	Xuint8 recv_data[RECV_CNT] = {0};
//	Xuint32 recv_cnt;
//	Xuint8 i;

	/************************ HARDWARE INITIALIZATION **************/
	//printf("\r\nBeginning Hardware Initialization\r\n");

	XStatus Status;
	

	// Initialize GPIO
	// GPIO_0: switches and push buttons
	//printf("\r\nBeginning Push Buttons Setup\r\n");
	Status = XGpio_Initialize(&SW_Gpio, XPAR_XPS_GPIO_0_DEVICE_ID);
	if (Status != XST_SUCCESS)
	{
		printf("GPIO_0 initialization error\n\r\r");
	}
	else
	{
		//printf("\r\nPush Buttons Setup Complete!\r\n\r\n");
	}
	/* Set the direction for all signals to be inputs */
	XGpio_SetDataDirection(&SW_Gpio, 1, 0xFFFFFFFF);

	// Initialize Video Decoder
	//printf("\r\nBeginning Video Setup\r\n");
	configDecoder(decoder_comp, DECODER_COMP_CONFIG_CNT);
	//printf("\r\nVideo Setup Complete!\r\n\r\n");

	// Initialize Audio
	//printf("\r\nBeginning Audio Setup\r\n");

	// Initialize GPIO for tone 1
	Status = XGpio_Initialize(&Audio1_Begin, XPAR_XPS_GPIO_1_DEVICE_ID);
	if (Status != XST_SUCCESS){
		printf("Audio begin initialization error\n\r\r");
	}
	XGpio_SetDataDirection(&Audio1_Begin, 1, 0);
	
	Status = XGpio_Initialize(&Audio1_Period, XPAR_XPS_GPIO_2_DEVICE_ID);
	if (Status != XST_SUCCESS){
		printf("Audio period initialization error\n\r\r");
	}
	XGpio_SetDataDirection(&Audio1_Period, 1, 0);

	Status = XGpio_Initialize(&Audio1_Length, XPAR_XPS_GPIO_3_DEVICE_ID);
	if (Status != XST_SUCCESS){
		printf("Audio length initialization error\n\r\r");
	}
	XGpio_SetDataDirection(&Audio1_Length, 1, 0);

	Status = XGpio_Initialize(&Audio1_Done, XPAR_XPS_GPIO_4_DEVICE_ID);
	if (Status != XST_SUCCESS){
		printf("Audio done initialization error\n\r\r");
	}
	XGpio_SetDataDirection(&Audio1_Done, 1, 0xFFFFFFFF);

	// Initialize GPIO for tone 2
	Status = XGpio_Initialize(&Audio2_Begin, XPAR_XPS_GPIO_5_DEVICE_ID);
	if (Status != XST_SUCCESS){
		printf("Audio begin initialization error\n\r\r");
	}
	XGpio_SetDataDirection(&Audio2_Begin, 1, 0);

	Status = XGpio_Initialize(&Audio2_Period, XPAR_XPS_GPIO_6_DEVICE_ID);
	if (Status != XST_SUCCESS){
		printf("Audio period initialization error\n\r\r");
	}
	XGpio_SetDataDirection(&Audio2_Period, 1, 0);

	Status = XGpio_Initialize(&Audio2_Length, XPAR_XPS_GPIO_7_DEVICE_ID);
	if (Status != XST_SUCCESS){
		printf("Audio length initialization error\n\r\r");
	}
	XGpio_SetDataDirection(&Audio2_Length, 1, 0);

	Status = XGpio_Initialize(&Audio2_Done, XPAR_XPS_GPIO_8_DEVICE_ID);
	if (Status != XST_SUCCESS){
		printf("Audio done initialization error\n\r\r");
	}
	XGpio_SetDataDirection(&Audio2_Done, 1, 0xFFFFFFFF);

	//printf("\r\nAudio Setup Complete!\r\n\r\n");

	// Initialize Timer
	//printf("\r\nBeginning Timer Setup\r\n");
	// Stop timer before initialization
	u32 ControlStatusReg = XTimerCtr_mReadReg(0x80040200, TIMER_COUNTER_0, XTC_TCSR_OFFSET);
	ControlStatusReg &= ~(XTC_CSR_ENABLE_TMR_MASK);
	XTmrCtr_mWriteReg(0x80040200, TIMER_COUNTER_0, XTC_TCSR_OFFSET, ControlStatusReg);
	// Timer setup
	Status = XTmrCtr_Initialize(&TimerCounter, XPAR_XPS_TIMER_0_DEVICE_ID);
	if (Status != XST_SUCCESS)
	{
		printf("Timer counter initialization error\n\r\r");
	}
	Status = XTmrCtr_SelfTest(&TimerCounter, TIMER_COUNTER_0);
	if (Status != XST_SUCCESS)
	{
		printf("Timer counter self-test error\n\r");
	}
	XTmrCtr_SetHandler(&TimerCounter, &TimerCounterHandler, &TimerCounter);
	XTmrCtr_SetOptions(&TimerCounter, TIMER_COUNTER_0, XTC_INT_MODE_OPTION | XTC_AUTO_RELOAD_OPTION);
	XTmrCtr_SetResetValue(&TimerCounter, TIMER_COUNTER_0, RESET_VALUE);
	// Do not start timer yet
	
	
	//printf("\r\nTimer Setup Complete!\r\n\r\n");

	// Initialize Interrupt Controller
	//printf("\r\nBeginning Interrupt Controller Setup\r\n");
	Status = XIntc_Initialize(&InterruptController, XPAR_XPS_INTC_0_DEVICE_ID);
	if (Status != XST_SUCCESS)
	{
		printf("Interrupt controller initialization error\n\r");
	}
	Status = XIntc_Connect( &InterruptController,
									XPAR_INTC_0_TMRCTR_0_VEC_ID,
									(XInterruptHandler)XTmrCtr_InterruptHandler,
									&TimerCounter);
	if (Status != XST_SUCCESS)
	{
		printf("Interrupt controller connect error\n\r");
	}
	Status = XIntc_Start(&InterruptController, XIN_REAL_MODE);
	if (Status != XST_SUCCESS)
	{
		printf("Interrupt controller start error\n\r");
	}
	//XIntc_Enable(&InterruptController, XPAR_INTC_0_TMRCTR_0_VEC_ID);
	microblaze_enable_interrupts();
	//printf("\r\nInterrupt Controller Setup Complete!\r\n\r\n");

	//printf("\r\nHardware Initialization Complete!\r\n\r\n");

	/************************ SOFTWARE INITIALIZATION **************/
	( (unsigned int *) SCREEN_SWITCH_BASEADDR)[0] = VIDEO_OUT_BASEADDR;
	//( (unsigned int *) SCREEN_SWITCH_BASEADDR)[0] = VIDEO_REPLACE_ADDR;

	( (unsigned int *) COLOR_LOWER_ADDR)[0] = COLOR_LOWER_BOUND_CALI;
	( (unsigned int *) COLOR_UPPER_ADDR)[0] = COLOR_UPPER_BOUND_CALI;
	( (unsigned int *) COLOR_REPLACE_ADDR)[0] = COLOR_REPLACED;


	// Display calibration start screen
	cali_start();
	// Wait until up is pushed
	while ((XGpio_DiscreteRead(&SW_Gpio, 1) & PUSH_BUTTON_UP) != 0) {}
	// Calibration
	showDetector();
	
	( (unsigned int *) COLOR_LOWER_ADDR)[0] = COLOR_LOWER_BOUND_BEACON;
	( (unsigned int *) COLOR_UPPER_ADDR)[0] = COLOR_UPPER_BOUND_BEACON;

	while(1){
		play();
		// Wait for button push to restart
		while((XGpio_DiscreteRead(&SW_Gpio, 1) & PUSH_BUTTON_UP) != 0) {delayTime(100);}
	}

	printf("done...");
	return 0;
}


void play()
{
	int exitFlag = 1;
	exitFlag = 0;
	snake_score = 0;
	Xuint32 initial_step_time = 300;
	Xuint32 difficulty_step = 60;
	//Xunit32 step_time = (initial_step_time > (snake_score / 3) * difficulty_step) ? (initial_step_time - (snake_score / 3) * difficulty_step) : (50);

	/************************ SOFTWARE START ***********************/
	//printf("start..");

	//showDetector();

	clear_screen();
	//printf("game start..");
	game_start();
	delayTime(3000);
	//play_score();
	play_gamestart();
	clear_screen();
	draw_box(10,10,630,470,1,COLOR_BLUE);
	snakeInit();
	delayTime(2000);
	//printf("game start..");

   while(!exitFlag)
   {
		exitFlag = update();
		draw_box(10,10,630,470,1,COLOR_BLUE);
		// Delay less when snake scores, as a 'level-up' difficulty adjustment
		//delayTime(5);
		delayTime((initial_step_time > (snake_score / 3) * difficulty_step) ? (initial_step_time - (snake_score / 3) * difficulty_step) : (5));
   }

	play_gameover();
	delayTime(1500);
	clear_screen();
	game_over();
}

////////////////////// From draw.c
/**
 * Paint Functions
 */
 
//Writes the given pixel value to the given screen at the given x-y location
void write_pixel(unsigned int value, unsigned int x, unsigned int y)
{
	( (unsigned int *) VIDEO_OUT_BASEADDR)[(y * 1024) + x] = value;
}

unsigned int get_pixel(unsigned int x, unsigned int y)
{
    return ( (unsigned int *) VIDEO_OUT_BASEADDR)[(y * 1024) + x];
}

int w_max( int a,  int b) {
	if (a > b)
		return a;
	else
		return b;
}

int w_min(int a,  int b) {
	if (a < b)
		return a;
	else
		return b;
}

unsigned int w_abs(int a) {
	if (a < 0)
		return -a;
	else
		return a;
}

void S(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour) {
	draw_thickLine(x - size, y - size, x + size, y - size, width, colour);
	draw_thickLine(x - size, y - size, x - size, y, width, colour);
	draw_thickLine(x - size, y, x + size, y, width, colour);
	draw_thickLine(x + size, y, x + size, y + size, width, colour);
	draw_thickLine(x - size, y + size, x + size, y + size, width, colour);
}

void T(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour) {
	draw_thickLine(x - size, y - size, x + size, y - size, width, colour);
	draw_thickLine(x, y - size, x, y + size, width, colour);
}

void A(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour) {
	draw_thickLine(x - size, y + size, x, y - size, width, colour);
	draw_thickLine(x + size, y + size, x, y - size, width, colour);
	draw_thickLine(x - size / 2, y, x + size / 2, y, width, colour);
}

void R(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour) {
	draw_thickLine(x - size, y + size, x - size, y - size, width, colour);
	draw_thickLine(x - size, y - size, x + size , y - size, width,
			colour);
	draw_thickLine(x + size , y - size, x + size , y, width,
			colour);
	draw_thickLine(x - size, y, x + size , y, width, colour);
	draw_thickLine(x - size, y, x + size , y + size, width, colour);
}

void G(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour) {
	draw_thickLine(x - size, y - size, x + size, y - size, width, colour);
	draw_thickLine(x - size, y + size, x - size, y - size, width, colour);
	draw_thickLine(x - size, y + size, x + size, y + size, width, colour);
	draw_thickLine(x + size, y + size, x + size, y, width, colour);
	draw_thickLine(x, y, x + size, y, width, colour);
}

void M(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour) {
	draw_thickLine(x - size, y - size, x - size, y + size, width, colour);
	draw_thickLine(x - size, y - size, x, y - size / 3, width, colour);
	draw_thickLine(x, y - size / 3, x + size, y - size, width, colour);
	draw_thickLine(x + size, y + size, x + size, y - size, width, colour);
}

void E(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour) {
	draw_thickLine(x - size, y - size, x - size, y + size, width, colour);
	draw_thickLine(x - size, y - size, x + size, y - size, width, colour);
	draw_thickLine(x - size, y, x + size, y, width, colour);
	draw_thickLine(x - size, y + size, x + size, y + size, width, colour);
}

void O(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour) {
	draw_thickLine(x - size, y - size, x - size, y + size, width, colour);
	draw_thickLine(x - size, y - size, x + size, y - size, width, colour);
	draw_thickLine(x+size, y-size, x + size,y + size, width, colour);
	draw_thickLine(x - size, y + size, x + size, y + size, width, colour);
}

void V(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour) {
	draw_thickLine(x - size, y - size, x, y + size, width, colour);
	draw_thickLine(x + size, y - size, x, y+size, width, colour);
}

void C(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour){
		
	draw_thickLine(x - size, y - size, x - size, y + size, width, colour);
	draw_thickLine(x - size, y - size, x + size, y - size, width, colour);
	draw_thickLine(x - size, y + size, x + size, y + size, width, colour);

}

void L(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour){
		
	draw_thickLine(x - size, y - size, x - size, y + size, width, colour);
	draw_thickLine(x - size, y + size, x + size, y + size, width, colour);
}

void I(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour){
		
    draw_thickLine(x, y - size, x, y + size, width, colour);
	draw_thickLine(x - size/2, y - size, x + size/2, y - size, width, colour);
	draw_thickLine(x - size/2, y + size, x + size/2, y + size, width, colour);
}

void B(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour){
	draw_thickLine(x - size, y - size, x - size, y + size, width, colour);
	draw_thickLine(x - size, y - size, x + size, y - size, width, colour);
	draw_thickLine(x - size, y + size, x + size, y + size, width, colour);	
	draw_thickLine(x + size, y - size, x + size, y + size, width, colour);	
    draw_thickLine(x - size, y , x + size, y , width, colour);	
}

void P(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour){
	draw_thickLine(x - size, y - size, x - size, y + size, width, colour);
	draw_thickLine(x - size, y - size, x + size, y - size, width, colour);
	draw_thickLine(x + size, y - size, x + size, y , width, colour);	
    draw_thickLine(x - size, y , x + size, y , width, colour);	
}

void N(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour){
	draw_thickLine(x - size, y - size, x - size, y + size, width, colour);
	draw_thickLine(x - size, y - size, x + size, y + size, width, colour);
	draw_thickLine(x + size, y - size, x + size, y + size, width, colour);	

}

void UARROW(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour){
	draw_thickLine(x, y - size, x, y + size, width, colour);
	draw_thickLine(x - size, y, x, y - size, width, colour);
	draw_thickLine(x + size, y, x, y - size, width, colour);
}

void DOT(unsigned int x, unsigned int y, unsigned int size, unsigned int width,
		unsigned int colour){
	draw_ball(x,y+size,width/2,colour);
}
		

/**
 * Draws a filled rectangle
 */
void draw_rectangle(unsigned int x1, unsigned int y1, unsigned int width,
		unsigned int length, const int colour) {
	int x_min, y_min, x_max, y_max;

	x_min = x1;
	y_min = y1;
	x_max = x1 + width;
	y_max = y1 + length;

	int x, y;

	for (y = y_min; y < y_max; y++) {
		for (x = x_min; x < x_max; x++) {
			write_pixel(colour, x, y);
		}
	}
}

void draw_box(int x0, int y0,int x1,int y1,int width, const int colour)
{
	draw_thickLine(x0, y0, x0, y1, width, colour);
	draw_thickLine(x0, y1, x1, y1, width, colour);
	draw_thickLine(x1, y1, x1, y0, width, colour);
	draw_thickLine(x1, y0, x0, y0, width, colour);
}

void draw_thickLine(int x0, int y0, int x1, int y1, int width, const int colour) {
	int dx = w_abs(x1 - x0);
	int dy = w_abs(y1 - y0);

	int a0, a1, i;
	if (dx > dy) {
		for (i = 0; i < width; i++) //edge along y
		{
			a0 = y0 + i;
			a1 = y1 + i;
			draw_line(x0, a0, x1, a1, colour);
		}
	} else {
		for (i = 0; i < width; i++) //edge along x
		{
			a0 = x0 + i;
			a1 = x1 + i;
			draw_line(a0, y0, a1, y1, colour);
		}
	}
}

void draw_niceLine(int x0, int y0, int x1, int y1, int width, const int colour)
{
		int a0, a1, i;

		for (i = 0; i < width/2; i++) //edge along y
		{
			a0 = y0 + i;
			a1 = y1 + i;
			draw_line(x0, a0, x1, a1, colour);
		}
		for (i = 0; i < width/2; i++) //edge along x
		{
			a0 = x0 + i;
			a1 = x1 + i;
			draw_line(a0, y0, a1, y1, colour);
		}
		draw_rectangle(x1,y1,width/2,width/2,colour);
}

void draw_ball(int X, int Y, int radius, int colour){
	int f = 1 - radius;
        int ddF_x = 1;
        int ddF_y = -2 * radius;

	draw_niceLine(X, Y - radius, X, Y + radius, 2, colour);

	int x = 0;
	int y = radius;

	while (x < y){
		if(f >= 0)
		{
			y--;
			ddF_y += 2;
			f += ddF_y;
		}
		x++;
		ddF_x += 2;
		f += ddF_x;
		draw_niceLine(X + x, Y + y, X + x, Y - y, 2,colour);
		draw_niceLine(X - x, Y + y, X - x, Y - y, 2,colour);
		draw_niceLine(X + y, Y + x, X + y, Y - x, 2,colour);
		draw_niceLine(X - y, Y + x, X - y, Y - x, 2,colour);
	}
}

void draw_line(int x0, int y0, int x1, int y1, const int colour) {
	int dx = w_abs(x1 - x0);
	int dy = w_abs(y1 - y0);

	int sx, sy, err;

	if (x0 < x1)
		sx = 1;
	else
		sx = -1;

	if (y0 < y1)
		sy = 1;
	else
		sy = -1;

	err = dx - dy;

	int x, y;
	int e2;
	x = x0;
	y = y0;
	while (true) {
		write_pixel(colour, x, y);

		if (x == x1 && y == y1)
			break;

		e2 = 2 * err;

		if (e2 > -dy) {
			err = err - dy;
			x = x + sx;
		}

		if (e2 < dx) {
			err = err + dx;
			y = y + sy;
		}
	}
}

void game_start() {
	draw_box(10,10,630,470,1,COLOR_BLUE);
	S(100, 200, 30, 2, COLOR_BLUE);
	T(200, 200, 30, 2, COLOR_BLUE);
	A(300, 200, 30, 2, COLOR_BLUE);
	R(400, 200, 30, 2, COLOR_BLUE);
	T(500, 200, 30, 2, COLOR_BLUE);
	G(140, 300, 30, 2, COLOR_BLUE);
	A(240, 300, 30, 2, COLOR_BLUE);
	M(340, 300, 30, 2, COLOR_BLUE);
	E(440, 300, 30, 2, COLOR_BLUE);
	
	
}

void game_over() {
	draw_box(10,10,630,470,1,COLOR_BLUE);
	G(150, 200, 30, 2, COLOR_BLUE);
	A(250, 200, 30, 2, COLOR_BLUE);
	M(350, 200, 30, 2, COLOR_BLUE);
	E(450, 200, 30, 2, COLOR_BLUE);
	O(150, 300, 30, 2, COLOR_BLUE);
	V(250, 300, 30, 2, COLOR_BLUE);
	E(350, 300, 30, 2, COLOR_BLUE);
	R(450, 300, 30, 2, COLOR_BLUE);
	
	P(200, 400, 6, 2, COLOR_BLUE);
	R(220, 400, 6, 2, COLOR_BLUE);
	E(240, 400, 6, 2, COLOR_BLUE);
	S(260, 400, 6, 2, COLOR_BLUE);
	S(280, 400, 6, 2, COLOR_BLUE);
	
	UARROW(330, 400, 6, 2, COLOR_BLUE);
	
	T(380, 400, 6, 2, COLOR_BLUE);
	O(400, 400, 6, 2, COLOR_BLUE);
	
	R(440, 400, 6, 2, COLOR_BLUE);
	E(460, 400, 6, 2, COLOR_BLUE);
	S(480, 400, 6, 2, COLOR_BLUE);
	T(500, 400, 6, 2, COLOR_BLUE);
	A(520, 400, 6, 2, COLOR_BLUE);
	R(540, 400, 6, 2, COLOR_BLUE);
	T(560, 400, 6, 2, COLOR_BLUE);
	
	DOT(580, 400, 6, 2, COLOR_BLUE);
	DOT(590, 400, 6, 2, COLOR_BLUE);
	DOT(600, 400, 6, 2, COLOR_BLUE);

}

void calibrate() {
    P(100, 200, 6, 2, COLOR_BLUE);
	R(120, 200, 6, 2, COLOR_BLUE);
	E(140, 200, 6, 2, COLOR_BLUE);
	S(160, 200, 6, 2, COLOR_BLUE);
	S(180, 200, 6, 2, COLOR_BLUE);
	
	UARROW(230, 200, 6, 2, COLOR_BLUE);
	
	T(280, 200, 6, 2, COLOR_BLUE);
	O(300, 200, 6, 2, COLOR_BLUE);
	
	C(340, 200, 6, 2, COLOR_BLUE);
	A(360, 200, 6, 2, COLOR_BLUE);
	L(380, 200, 6, 2, COLOR_BLUE);
	I(400, 200, 6, 2, COLOR_BLUE);
	B(420, 200, 6, 2, COLOR_BLUE);
	R(440, 200, 6, 2, COLOR_BLUE);
	A(460, 200, 6, 2, COLOR_BLUE);
	T(480, 200, 6, 2, COLOR_BLUE);
	E(500, 200, 6, 2, COLOR_BLUE);
	DOT(520, 200, 6, 2, COLOR_BLUE);
	DOT(530, 200, 6, 2, COLOR_BLUE);
	DOT(540, 200, 6, 2, COLOR_BLUE);
}

void calibrating(){
	C(220, 200, 6, 2, COLOR_BLUE);
	A(240, 200, 6, 2, COLOR_BLUE);
	L(260, 200, 6, 2, COLOR_BLUE);
	I(280, 200, 6, 2, COLOR_BLUE);
	B(300, 200, 6, 2, COLOR_BLUE);
	R(320, 200, 6, 2, COLOR_BLUE);
	A(340, 200, 6, 2, COLOR_BLUE);
	T(360, 200, 6, 2, COLOR_BLUE);
	I(380, 200, 6, 2, COLOR_BLUE);
	N(400, 200, 6, 2, COLOR_BLUE);
	G(420, 200, 6, 2, COLOR_BLUE);
	DOT(440, 200, 6, 2, COLOR_BLUE);
	DOT(450, 200, 6, 2, COLOR_BLUE);
	DOT(460, 200, 6, 2, COLOR_BLUE);
	
	P(120, 300, 6, 2, COLOR_BLUE);
	L(140, 300, 6, 2, COLOR_BLUE);
	A(160, 300, 6, 2, COLOR_BLUE);
	C(180, 300, 6, 2, COLOR_BLUE);
	E(200, 300, 6, 2, COLOR_BLUE);
	
	B(240, 300, 6, 2, COLOR_BLUE);
	E(260, 300, 6, 2, COLOR_BLUE);
	A(280, 300, 6, 2, COLOR_BLUE);
	C(300, 300, 6, 2, COLOR_BLUE);
	O(320, 300, 6, 2, COLOR_BLUE);
	N(340, 300, 6, 2, COLOR_BLUE);
	
	O(380, 300, 6, 2, COLOR_BLUE);
	N(400, 300, 6, 2, COLOR_BLUE);
	
	R(440, 300, 6, 2, COLOR_BLUE);
	E(460, 300, 6, 2, COLOR_BLUE);
	O(480, 300, 6, 2, COLOR_BLUE);
	
	
}

void cali_start() {
	// For determining color bound:
	// Set screen to be red, then output whatever captured by camera to hyperterminal
//	red_screen();
//	unsigned int pixel_read;
//	int i;
//	for (i = 0; ; i++){
//		delayTime(3000);
//		pixel_read = ( (unsigned int *) VIDEO_IN_BASEADDR)[(100 * 1024) + 100];
//		xil_printf("%x\n", pixel_read);
//	}

	// For regular calibration:
	clear_screen();
	calibrate();
	
}

void clear_screen() {
	int x;
	int y;
	for (y = 0; y < 480; y++) {
		for (x = 0; x < 640; x++) {
			write_pixel(COLOR_WHITE, x, y);
		}
	}
}

void red_screen() {
	int x;
	int y;
	for (y = 0; y < 480; y++) {
		for (x = 0; x < 640; x++) {
			write_pixel(COLOR_RED, x, y);
		}
	}
}

void green_screen() {
	int x;
	int y;
	for (y = 0; y < 480; y++) {
		for (x = 0; x < 640; x++) {
			write_pixel(COLOR_GREEN, x, y);
		}
	}
}

void blue_screen() {
	int x;
	int y;
	for (y = 0; y < 480; y++) {
		for (x = 0; x < 640; x++) {
			write_pixel(COLOR_BLUE, x, y);
		}
	}
}

////////////////////// From snake.c
int transformToX(int u)
{

    if(u==0)
		return u;
	
	int x_width = detectorX[1] - detectorX[0];

	int x = (u - detectorU[0])*x_width/deltaU + detectorX[0];

	x = w_max(x,10);
	x = w_min(x,630);
	return x;
}

int transformToY(int v)
{
	if(v==0)
		return v;
		
	int y_height = detectorY[2] - detectorY[0];
	int y = (v - detectorV[0])*y_height/deltaV + detectorY[0];

	y = w_max(y,10);
	y = w_min(y,470);
	return y;
}

unsigned int getUSum()
{
   return ( (unsigned int *) BEACON_X_ADDR)[0];
}

unsigned int getVSum()
{
   return ( (unsigned int *) BEACON_Y_ADDR)[0];
}

unsigned int getCount()
{
   return ( (unsigned int *) BEACON_COUNT_ADDR)[0];
}

int calculateUV(int sum, int count)
{
	int uv;
	
   if(count==0)
     return 0;
	  
	uv = sum/count;
	return uv;
}

void showDetector()
{
	xil_printf("show detector\n");
	
	int i=0;
	int u,v, count;
	
	clear_screen();
	calibrating();
	
	for(i=0;i<4;i++)
	{
		draw_ball(detectorX[i],detectorY[i],10,COLOR_RED);
		delayTime(7000);
		u = getUSum();
		v = getVSum();
		count = getCount();
		delayTime(3000);
		
		detectorU[i]=calculateUV(u,count);
		detectorV[i]=calculateUV(v,count);

		xil_printf("x: %d, y: %d\n",detectorU[i],detectorV[i]);
		draw_ball(detectorX[i],detectorY[i],10,COLOR_WHITE);
	}

	/*
	detectorU[0]=100;
	detectorV[0]=100;
	detectorU[1]=300;
	detectorV[1]=100;
	detectorU[2]=300;
	detectorV[2]=300;
	detectorU[3]=100;
	detectorV[3]=300;

	*/
   deltaU = detectorU[1] - detectorU[0];
   deltaV = detectorV[2] - detectorV[0];
   
   xil_printf("deltaU: %d, deltaV: %d\n",deltaU,deltaV);
	
	if(deltaU==0 || deltaV==0)
	{
		deltaU = detectorX[1] - detectorX[0];
		deltaV = detectorY[2] - detectorY[0];
	   
	   	detectorU[0]=detectorX[0];
		detectorV[0]=detectorY[0];
		detectorU[1]=detectorX[1];
		detectorV[1]=detectorY[1];
		detectorU[2]=detectorX[2];
		detectorV[2]=detectorY[2];
		detectorU[3]=detectorX[3];
		detectorV[3]=detectorY[3];
	}	
	


	/*transformToX(detectorU[0]);
	transformToY(detectorV[0]);*/

	//std::cout <<"transformed:"<<transformToX(10)<<","<<transformToY(detectorV[0]) << std::endl;
}

void setBeacon(int u, int v)
{
	
	
	if(u!=0 && v!=0)
	{
	   draw_ball(beaconX,beaconY,3,COLOR_WHITE);
		//beaconX=u;
		//beaconY=v;
		beaconX = transformToX(u);
		beaconY = transformToY(v);
		
		xil_printf("u:%d, v:%d\n",u,v);
		xil_printf("beacon:%d,%d\n",beaconX,beaconY);
		//std::cout<<"Beacon: "<<beaconX<<","<<beaconY << std::endl;
		draw_ball(beaconX,beaconY,3,COLOR_YELLOW);
   }
	
}

/**
 * Snake functions
 */

void generateFood()
{
    float randnum;

    randnum = rand()/(double)RAND_MAX;

    foodX = 40 + randnum*(SCREEN_WIDTH-80);

    randnum = rand()/(double)RAND_MAX;

    foodY = 40 + randnum*(SCREEN_HEIGHT-80);

    //std::cout <<foodX<<","<<foodY << std::endl;

    draw_ball(foodX, foodY,5,COLOR_GREEN);
}


void drawSnakePoint(SnakePoint* point, int color) {
	draw_ball(point->x0, point->y0,5,color);
	draw_thickLine(point->x0, point->y0, point->x1, point->y1, 5, color);
	draw_ball(point->x1, point->y1,5,color);
}

void drawSnake(SnakePoint * headPointer) {
	SnakePoint * nextPtr;

	nextPtr = headPointer;
	drawSnakePoint(nextPtr,COLOR_BLUE);
	//std::cout << "==="<< std::endl;
	while(1)
	{
		nextPtr = nextPtr->next;
		drawSnakePoint(nextPtr,COLOR_BLUE);
		//std::cout << nextPtr->x0<<","<<nextPtr->y0<<std::endl;
		if(nextPtr->end==1)
			break;
	}
	//std::cout << "==="<< std::endl;
}

void snakeEat(int incrX, int incrY)
{
	draw_ball(foodX, foodY,5,COLOR_WHITE);
	SnakePoint * newHeadPtr =(SnakePoint *) malloc(sizeof(SnakePoint));
	newHeadPtr->x0 = headPtr->x1;
	newHeadPtr->y0 = headPtr->y1;
	newHeadPtr->x1 = headPtr->x1 + incrX;
	newHeadPtr->y1 = headPtr->y1 + incrY;
	newHeadPtr->end = 0;
	newHeadPtr->next =  headPtr;

	headPtr = newHeadPtr;

	//drawSnake(headPtr);
	drawSnakePoint(headPtr,COLOR_BLUE);
}

int detectFront(int x, int y, int deltaX, int deltaY, unsigned int color)
{
   int i,j;
   int min_x,min_y,max_x,max_y;

   min_x = w_min(x,x+deltaX);
   max_x = w_max(x,x+deltaX);

   min_y = w_min(y,y+deltaY);
   max_y = w_max(y,y+deltaY);

   //std::cout <<"Detect Front:"<<x<<","<<y<< std::endl;
   //std::cout <<"Detect Front:"<<min_x<<","<<min_y<< std::endl;
   //std::cout <<"Detect Front:"<<deltaX<<","<<deltaY<< std::endl;

   for(i=min_x;i<=max_x;i++)
	 {
	   for(j=min_y;j<=max_y;j++)
	   {
		   if(get_pixel(i,j)==color)
			   return 1;
	   }
	 }
   return 0;
}

int detectFood(int x, int y)
{
	int range = 10;
	if((foodX-range)<x && x<(foodX+range))
	{
		if((foodY-range)<y && y<(foodY+range))
		{
			return 1;
		}
	}
	return 0;
}

int detectBeacon(int x, int y)
{
	int range = 5;

	if((beaconX-range)<x && x<(beaconX+range))
	{
		if((beaconY-range)<y && y<(beaconY+range))
		{
			return 1;
		}
	}
	return 0;
}

void setIncr(int deltaX,int deltaY,int * incrX, int* incrY)
{
    int hypotenuse = sqrt(deltaX*deltaX + deltaY*deltaY);

    //prevent hypotenuse from equal to zero
    hypotenuse = 1 + hypotenuse;

    *incrX = (deltaX*10)/hypotenuse;
    *incrY = (deltaY*10)/hypotenuse;
}

int snakeMove(int incrX, int incrY)
{
	SnakePoint currPoint;
	SnakePoint nextPoint;
	SnakePoint endPoint;

	currPoint = *headPtr;
	endPoint = *tailPtr;

	int lostFlag=0;

	if(detectBeacon(currPoint.x1,currPoint.y1)==1)
	{
		//std::cout <<"beacon!"<< std::endl;
		return 0;
	}

	/**
	 * Detect whether the snake has hit the border or itself
	 */
	//std::cout <<currPoint.x1+incrX<<","<<currPoint.y1+incrY<< std::endl;
	if(detectFront(currPoint.x1+incrX,currPoint.y1+incrY, incrX,incrY,COLOR_BLUE))
	{
		//std::cout <<"lost!!!!!!!!!"<< std::endl;
		lostFlag=1;
	}


	/*
	 * Detect whether the snake has reached the food
	 */

	if(detectFood(currPoint.x1,currPoint.y1))
	{
		//std::cout <<"eat!"<< std::endl;
		snakeEat(incrX,incrY);
      play_score();
		snake_score++;
		generateFood();
		return 0;
	}

	/*drawSnakePoint(tailPtr,COLOR_WHITE);*/

	while(currPoint.end!=1)
	{
		nextPoint = *(currPoint.next); //Backup the value of next item in cache

		//std::cout <<"move!!!"<< std::endl;
		//value of next item becomes current
		currPoint.next->x0 = currPoint.x0;
		currPoint.next->y0 = currPoint.y0;
		currPoint.next->x1 = currPoint.x1;
		currPoint.next->y1 = currPoint.y1;

		currPoint = nextPoint;


	}

	headPtr->x0 = headPtr->x1;
	headPtr->y0 = headPtr->y1;
	headPtr->x1 = headPtr->x0 + incrX;
	headPtr->y1 = headPtr->y0 + incrY;
	
    drawSnakePoint(&endPoint,COLOR_WHITE);
	drawSnake(headPtr);
	//drawSnakePoint(headPtr,COLOR_BLUE);
	//drawSnakePoint(&endPoint,COLOR_WHITE);

	return lostFlag;
}



void snakeInit()
{
	headPtr = (SnakePoint *) malloc(sizeof(SnakePoint));
	tailPtr = (SnakePoint *) malloc(sizeof(SnakePoint));

	tailPtr->x0 = 90;
	tailPtr->y0 = 100;
	tailPtr->x1 = 100;
	tailPtr->y1 = 100;
	tailPtr->end=1;
	tailPtr->next=0;

	headPtr->x0 = 100;
	headPtr->y0 = 100;
	headPtr->x1 = 110;
	headPtr->y1 = 100;
	headPtr->end=0;
	headPtr->next = tailPtr;

	//srand(time(0));
	generateFood();


    incrX=10;
    incrY=0;

	beaconX = 600;
	beaconY = 100;
    int deltaX, deltaY;
    deltaX = beaconX - headPtr->x1;
    deltaY = beaconY - headPtr->y1;

    setIncr(deltaX,deltaY,&incrX,&incrY);
}

int update()
{
    //sleep(1);
	int deltaX,deltaY,exitflag;
	int u,v, count;
	exitflag=0;

	//Getting beacon location

	u = getUSum();
	v = getVSum();
	count = getCount();

	

	u=calculateUV(u,count);
	v=calculateUV(v,count);

	

	setBeacon(u,v);

	deltaX = beaconX -headPtr->x1;
	deltaY = beaconY - headPtr->y1;

	setIncr(deltaX,deltaY,&incrX,&incrY);

	//std::cout <<"incre: "<<incrX<<","<<incrY<<std::endl;

	exitflag = snakeMove(incrX,incrY);

	return exitflag;
}

void delayTime(Xuint32 ms_to_delay)
{
	TimerExpiredCount = 0;
	// Start Timer
	XIntc_Enable(&InterruptController, XPAR_INTC_0_TMRCTR_0_VEC_ID);
	XTmrCtr_Reset(&TimerCounter, TIMER_COUNTER_0);
	XTmrCtr_Start(&TimerCounter, TIMER_COUNTER_0);
	while (TimerExpiredCount < ms_to_delay) {}
	// Stop Timer
	XTmrCtr_Stop(&TimerCounter, TIMER_COUNTER_0);
	XIntc_Disable(&InterruptController, XPAR_INTC_0_TMRCTR_0_VEC_ID);
}

////////////////////// From video_setup.c
void configDecoder(struct VideoModule *decoder, int config_cnt ) 
{
	Xuint16 send_cnt, i;
	Xuint8 send_data[2] = {0};
	Xuint8 success = 1;
	send_cnt = 2;
	printf("  Configuring Decoder...\t");
	//read in configuration constants and send them to the VDEC one by one
	for( i = 0; i < config_cnt; i++ )
	{
		send_data[0] = decoder[i].addr;

		send_data[1] = decoder[i].config_val;

		send_cnt = XIic_Send(XPAR_IIC_BASEADDR, DECODER_ADDR, send_data, 2,XIIC_STOP );

		printf("\nWriting to address %02x send cnt:%d", decoder[i].addr, send_cnt);

		if( send_cnt != 2 ) 
		{
			printf("Error writing to address %02x\r\n", decoder[i].addr);
			success = 0;
			break;
		}
	}

	if( success != 0) {
		printf("\r\nSUCCESS!\r\n");
	}
	else {
		printf("\r\nFailed!\r\n"); 
	}
}

////////////////////// From audio_setup.c
//Function for creating a tone using audio hardware
void makeTone(int channel, u32 pitch, u32 length, int async){
	XGpio* pBegin = channel==1?(&Audio1_Begin):(&Audio2_Begin);
	XGpio* pPeriod = channel==1?(&Audio1_Period):(&Audio2_Period);
	XGpio* pLength = channel==1?(&Audio1_Length):(&Audio2_Length);
	XGpio* pDone = channel==1?(&Audio1_Done):(&Audio2_Done);

	if(XGpio_DiscreteRead(pDone, 1))
	{	
		XGpio_DiscreteWrite(pPeriod, 1, pitch);
		XGpio_DiscreteWrite(pLength, 1, length);
		XGpio_DiscreteWrite(pBegin, 1, 1);
		XGpio_DiscreteWrite(pBegin, 1, 0);

		if(!async)
		{
			while(!XGpio_DiscreteRead(pDone, 1)) {}
		}
	}
}

void play_gamestart(){
	// Test different tones
	makeTone(2,PERIOD_E,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR*3,0);
	makeTone(2,PERIOD_E,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR*3,0);
	makeTone(2,PERIOD_E,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR*3,0);
	makeTone(2,PERIOD_C,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR*3,0);
	makeTone(2,PERIOD_E,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR*3,0);
	makeTone(2,PERIOD_G,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR*4,0);
	makeTone(2,PERIOD_G,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR*4,0);
    makeTone(2,PERIOD_C/2,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR*4,0);
		
		
  /*
	makeTone(2,PERIOD_C,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_D,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_E,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_F,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_G,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_A,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_B,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);

	makeTone(2,PERIOD_C/2,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_D/2,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_E/2,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_F/2,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_G/2,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_A/2,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_B/2,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
  */
}

void play_score(){
	makeTone(2,PERIOD_E/2,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_G/2,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_E/4,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_C/4,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_D/4,TONE_DUR,0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR,0);
	makeTone(2,PERIOD_G/4,TONE_DUR*1.2,0);
}

void play_gameover(){
	makeTone(2, 1.5*PERIOD_G, GAME_OVER_DUR, 0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR*3,0);
	makeTone(2, 1.5*PERIOD_C/2, GAME_OVER_DUR, 0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR*3,0);
	makeTone(2, 1.5*PERIOD_D/2, GAME_OVER_DUR, 0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR*3,0);
	makeTone(2, 1.5*PERIOD_Ds/2, GAME_OVER_DUR*4, 0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR*3,0);
	makeTone(2, 1.5*PERIOD_D/2, GAME_OVER_DUR*2, 0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR*3,0);
	makeTone(2, 1.5*PERIOD_C/2, GAME_OVER_DUR*1.5, 0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR*3,0);
	makeTone(2, 1.5*PERIOD_As, GAME_OVER_DUR*0.5, 0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR*3,0);
	makeTone(2, 1.5*PERIOD_C/2, GAME_OVER_DUR*4, 0);
		makeTone(2,0xFFFFFFFF,PAUSE_DUR*3,0);
//	makeTone(1, PERIOD_B,GAME_OVER_DUR, 0);
//
//	makeTone(2, 0xFFFFFFFF, GAME_OVER_DUR/2, 0);
//	makeTone(2, PERIOD_D/2, GAME_OVER_DUR/2, 0);
//
//	makeTone(1, PERIOD_As, GAME_OVER_DUR, 1);
//	makeTone(2, PERIOD_Cs/2, GAME_OVER_DUR/2, 0);
//	makeTone(2, 0xFFFFFFFF, GAME_OVER_DUR/8, 0);
//	makeTone(2, PERIOD_Cs/2, GAME_OVER_DUR/2, 0);	
//	makeTone(2, 0xFFFFFFFF, GAME_OVER_DUR/8, 0);
//	makeTone(2, PERIOD_D/2, GAME_OVER_DUR/2, 0);
//
//	makeTone(2, 0xFFFFFFFF, GAME_OVER_DUR/8, 0);	
//	makeTone(1, PERIOD_A, GAME_OVER_DUR, 1);	
//	makeTone(2, PERIOD_D/2, GAME_OVER_DUR/2, 0);	
//	makeTone(2, 0xFFFFFFFF, GAME_OVER_DUR/8, 0);
//	makeTone(2, PERIOD_D/2, GAME_OVER_DUR/2, 0);	
//	makeTone(2, 0xFFFFFFFF, GAME_OVER_DUR/8, 0);
//	makeTone(2, PERIOD_Ds/2, GAME_OVER_DUR/2, 0);	
//	makeTone(2, 0xFFFFFFFF, GAME_OVER_DUR/8, 0);
//
//	makeTone(1, PERIOD_G, GAME_OVER_DUR, 1);		
//	makeTone(2, PERIOD_Ds/2, GAME_OVER_DUR/2, 0);	
//	makeTone(2, 0xFFFFFFFF, GAME_OVER_DUR/8, 0);	
//	makeTone(2, PERIOD_E/2, GAME_OVER_DUR/2, 0);	
//	makeTone(2, 0xFFFFFFFF, GAME_OVER_DUR/8, 0);	
//	makeTone(2, PERIOD_Fs/2, GAME_OVER_DUR/2, 0);	
//	makeTone(2, 0xFFFFFFFF, GAME_OVER_DUR/8, 0);
//	makeTone(1, PERIOD_E, GAME_OVER_DUR, 1);
//	makeTone(2, PERIOD_G/2, GAME_OVER_DUR, 0);	
//
//	makeTone(2, 0xFFFFFFFF, GAME_OVER_DUR*2, 0);
//
//	makeTone(1, PERIOD_G, GAME_OVER_DUR, 1);	
//	makeTone(1, PERIOD_E, GAME_OVER_DUR, 0);
//
//	makeTone(2, PERIOD_C/2, GAME_OVER_DUR, 0);	

	
}

void TimerCounterHandler(void *CallBackRef, Xuint8 TimerCounterNumber)
{
	TimerExpiredCount++;
}
